# frozen_string_literal: true

require 'parslet'

module MNC
  class Parslet < Parslet::Parser
    rule(:crontab)     { lines.as(:crontab) }
    rule(:lines)       { (line >> eol).repeat }
    rule(:line)        { cron | env | comment | blank }

    rule(:space)       { match[" \t"].repeat(1) }
    rule(:eol)         { line_end.repeat(1) }
    rule(:line_end)    { crlf >> space.maybe }
    rule(:crlf)        { match["\r\n"].repeat(1) }
    rule(:eof)         { any.absent? }

    rule(:blank)       { space.repeat }

    rule(:comment)     { space >> hash >> match["^\n"].repeat }
    rule(:hash)        { str('#') }

    rule(:env)         { variable >> space >> equal >> space >> value }
    rule(:variable)    { match('\w').repeat(1) }
    rule(:equal)       { str('=') }
    rule(:value)       { match["^\n"].repeat(1) }

    rule(:cron)        { (special | traditional).as(:cron) >> space >> command }
    rule(:special)     { (str('@') >> match['a-z'].repeat(1)).as(:special) }
    rule(:traditional) { minute >> space >> hour >> space >> dom >> space >> month >> space >> dow }

    rule(:minute)      { entry.as(:minute) }
    rule(:hour)        { entry.as(:hour) }
    rule(:dom)         { entry.as(:dom) }
    rule(:month)       { entry.as(:month) }
    rule(:dow)         { entry.as(:dow) }

    rule(:entry)       { step | wildcard | list }
    rule(:step)        { str('*/') >> integer }
    rule(:wildcard)    { str('*') }
    rule(:list)        { (range | integer) >> (str(',') >> (range | integer)).repeat }
    rule(:range)       { integer >> str('-') >> integer }
    rule(:integer)     { match['0-9'].repeat(1) }
    rule(:command)     { match["^\n"].repeat(1) }

    root(:crontab)
  end
end
